//+------------------------------------------------------------------+
//|                                       PositionSizeCalculator.mq5 |
//|                             Copyright  2012-2013, Andriy Moraru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012-2013, Andriy Moraru"
#property link      "http://www.earnforex.com "
#property version   "1.8"

#property description "Calculates position size based on account balance/equity,"
#property description "currency, currency pair, given entry level, stop-loss level"
#property description "and risk tolerance (set either in percentage points or in base currency)."
#property description "2013-02-11, ver. 1.8 - completely revamped calculation process."
#property description "2013-01-14, ver. 1.7 - fixed \"division by zero\" error."
#property description "2012-12-10, ver. 1.6 - will use local values if both Entry and SL are missing."
#property description "2012-11-02, ver. 1.5 - a more intelligent name prefix/postfix detection."
#property description "2012-10-13, ver. 1.4 - fixed contract size in lot size calculation."
#property description "2012-10-13, ver. 1.3 - proper lot size calculation for gold, silver and oil."
#property description "2012-09-29, ver. 1.2 - improved account currency and reference pair detection."
#property description "2012-05-10, ver. 1.1 - added support for setting risk in money."

#property indicator_chart_window

input double EntryLevel = 0;
input double StopLossLevel = 0;
input double Risk = 1; // Risk tolerance in percentage points
input double MoneyRisk = 0; // Risk tolerance in base currency
input bool UseMoneyInsteadOfPercentage = false;
input bool UseEquityInsteadOfBalance = false;
input bool DeleteLines = false; // DeleteLines - If true, will delete lines on deinitialization. Otherwise will leave lines, so levels can be restored.

input color font_color = clrLightBlue;
input color ps_font_color = clrRed;
input int font_size = 12;
input string font_face = "Courier";
input ENUM_BASE_CORNER corner = CORNER_LEFT_UPPER;
input int distance_x = 10;
input int distance_y = 15;
input color entry_line_color = clrBlue;
input color stoploss_line_color = clrLime;

string SizeText;
double Size, RiskMoney;
double PositionSize;
double StopLoss;

// Will be used instead of input parameters as they cannot be modified during runtime.
double iEntryLevel;
double iStopLossLevel;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
void OnInit()
{
   if (ObjectFind(0, "EntryLine") > -1) iEntryLevel = ObjectGetDouble(0, "EntryLine", OBJPROP_PRICE);
   else iEntryLevel = EntryLevel;
   if (ObjectFind(0, "StopLossLine") > -1) iStopLossLevel = ObjectGetDouble(0, "StopLossLine", OBJPROP_PRICE);
   else iStopLossLevel = StopLossLevel;

   if ((iEntryLevel == 0) && (iStopLossLevel == 0))
   {
      Print(Symbol() + ": Entry and Stop-Loss levels not given. Using local values.");
      iEntryLevel = SymbolInfoDouble(Symbol(), SYMBOL_BIDHIGH);
      iStopLossLevel = SymbolInfoDouble(Symbol(), SYMBOL_BIDLOW);
      if (iEntryLevel == iStopLossLevel) iStopLossLevel -= _Point;
   }

   if (iEntryLevel - iStopLossLevel == 0)
   {
      Alert("Entry and Stop-Loss levels should be different and non-zero.");
      return;
   }

   ObjectCreate(0, "Entry", OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, "Entry", OBJPROP_CORNER, corner);
   ObjectSetInteger(0, "Entry", OBJPROP_XDISTANCE, distance_x);
   ObjectSetInteger(0, "Entry", OBJPROP_YDISTANCE, distance_y);
   ObjectSetString(0, "Entry", OBJPROP_TEXT, "Entry Lvl:    " + DoubleToString(iEntryLevel, _Digits));
   ObjectSetInteger(0, "Entry", OBJPROP_FONTSIZE, font_size);
   ObjectSetString(0, "Entry", OBJPROP_FONT, font_face);
   ObjectSetInteger(0, "Entry", OBJPROP_COLOR, font_color);

   if (ObjectFind(0, "EntryLine") == -1)
   {
      ObjectCreate(0, "EntryLine", OBJ_HLINE, 0, TimeCurrent(), iEntryLevel);
      ObjectSetInteger(0, "EntryLine", OBJPROP_STYLE, STYLE_SOLID);
      ObjectSetInteger(0, "EntryLine", OBJPROP_COLOR, entry_line_color);
      ObjectSetInteger(0, "EntryLine", OBJPROP_WIDTH, 1);
      ObjectSetInteger(0, "EntryLine", OBJPROP_SELECTABLE, true);
   }
   
   ObjectCreate(0, "StopLoss", OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, "StopLoss", OBJPROP_CORNER, corner);
   ObjectSetInteger(0, "StopLoss", OBJPROP_XDISTANCE, distance_x);
   ObjectSetInteger(0, "StopLoss", OBJPROP_YDISTANCE, distance_y + 15);
   ObjectSetString(0, "StopLoss", OBJPROP_TEXT, "Stop-Loss:    " + DoubleToString(iStopLossLevel, _Digits));
   ObjectSetInteger(0, "StopLoss", OBJPROP_FONTSIZE, font_size);
   ObjectSetString(0, "StopLoss", OBJPROP_FONT, font_face);
   ObjectSetInteger(0, "StopLoss", OBJPROP_COLOR, font_color);

   if (ObjectFind(0, "StopLossLine") == -1)
   {
      ObjectCreate(0, "StopLossLine", OBJ_HLINE, 0, TimeCurrent(), iStopLossLevel);
      ObjectSetInteger(0, "StopLossLine", OBJPROP_STYLE, STYLE_SOLID);
      ObjectSetInteger(0, "StopLossLine", OBJPROP_COLOR, stoploss_line_color);
      ObjectSetInteger(0, "StopLossLine", OBJPROP_WIDTH, 1);
      ObjectSetInteger(0, "StopLossLine", OBJPROP_SELECTABLE, true);
   }
   
   StopLoss = MathAbs(iEntryLevel - iStopLossLevel);
   
   if (!UseMoneyInsteadOfPercentage)
   {
      ObjectCreate(0, "Risk", OBJ_LABEL, 0, 0, 0);
      ObjectSetInteger(0, "Risk", OBJPROP_CORNER, corner);
      ObjectSetInteger(0, "Risk", OBJPROP_XDISTANCE, distance_x);
      ObjectSetInteger(0, "Risk", OBJPROP_YDISTANCE, distance_y + 30);
      ObjectSetString(0, "Risk", OBJPROP_TEXT, "Risk:         " + DoubleToString(Risk, 2) + "%");
      ObjectSetInteger(0, "Risk", OBJPROP_FONTSIZE, font_size);
      ObjectSetString(0, "Risk", OBJPROP_FONT, font_face);
      ObjectSetInteger(0, "Risk", OBJPROP_COLOR, font_color);
   }
   
   if (UseEquityInsteadOfBalance)
   {
      SizeText = "Equity";
      Size = AccountInfoDouble(ACCOUNT_EQUITY);
   }
   else
   {
      SizeText = "Balance";
      Size = AccountInfoDouble(ACCOUNT_BALANCE);
   }
   ObjectCreate(0, "AccountSize", OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, "AccountSize", OBJPROP_CORNER, corner);
   ObjectSetInteger(0, "AccountSize", OBJPROP_XDISTANCE, distance_x);
   ObjectSetInteger(0, "AccountSize", OBJPROP_YDISTANCE, distance_y + 45);
   ObjectSetString(0, "AccountSize", OBJPROP_TEXT, "Acc. " + SizeText + ": " + DoubleToString(Size, 2));
   ObjectSetInteger(0, "AccountSize", OBJPROP_FONTSIZE, font_size);
   ObjectSetString(0, "AccountSize", OBJPROP_FONT, font_face);
   ObjectSetInteger(0, "AccountSize", OBJPROP_COLOR, font_color);
   
   ObjectCreate(0, "RiskMoney", OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, "RiskMoney", OBJPROP_CORNER, corner);
   ObjectSetInteger(0, "RiskMoney", OBJPROP_XDISTANCE, distance_x);
   ObjectSetInteger(0, "RiskMoney", OBJPROP_YDISTANCE, distance_y + 60);
   ObjectSetInteger(0, "RiskMoney", OBJPROP_FONTSIZE, font_size);
   ObjectSetString(0, "RiskMoney", OBJPROP_FONT, font_face);
   ObjectSetInteger(0, "RiskMoney", OBJPROP_COLOR, font_color);
   
   ObjectCreate(0, "PositionSize", OBJ_LABEL, 0, 0, 0);
   ObjectSetInteger(0, "PositionSize", OBJPROP_CORNER, corner);
   ObjectSetInteger(0, "PositionSize", OBJPROP_XDISTANCE, distance_x);
   ObjectSetInteger(0, "PositionSize", OBJPROP_YDISTANCE, distance_y + 75);
   ObjectSetInteger(0, "PositionSize", OBJPROP_FONTSIZE, font_size + 1);
   ObjectSetString(0, "PositionSize", OBJPROP_FONT, font_face);
   ObjectSetInteger(0, "PositionSize", OBJPROP_COLOR, ps_font_color);

   if (AccountInfoString(ACCOUNT_CURRENCY) == "") return;

   CalculateRiskAndPositionSize();
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   ObjectDelete(0, "Entry");
   if (DeleteLines) ObjectDelete(0, "EntryLine");
   ObjectDelete(0, "StopLoss");
   if (DeleteLines) ObjectDelete(0, "StopLossLine");
   if (!UseMoneyInsteadOfPercentage) ObjectDelete(0, "Risk"); // Otherwise wasn't created.
   ObjectDelete(0, "AccountSize");
   ObjectDelete(0, "RiskMoney");
   ObjectDelete(0, "PositionSize");
}

//+------------------------------------------------------------------+
//| Main recalculation function used on every tick and on entry/SL   |
//| line drag                                                        |
//+------------------------------------------------------------------+
void RecalculatePositionSize()
{
   if (iEntryLevel - iStopLossLevel == 0) return;
   
   // If could not find account currency, probably not connected.
   if (AccountInfoString(ACCOUNT_CURRENCY) == "") return;
   
   double tEntryLevel = ObjectGetDouble(0, "EntryLine", OBJPROP_PRICE);
   double tStopLossLevel = ObjectGetDouble(0, "StopLossLine", OBJPROP_PRICE);
   ObjectSetString(0, "Entry", OBJPROP_TEXT, "Entry Lvl:    " + DoubleToString(tEntryLevel, _Digits));
   ObjectSetString(0, "StopLoss", OBJPROP_TEXT, "Stop-Loss:    " + DoubleToString(tStopLossLevel, _Digits));

   StopLoss = MathAbs(tEntryLevel - tStopLossLevel);

   if (UseEquityInsteadOfBalance) Size = AccountInfoDouble(ACCOUNT_EQUITY);
   else Size = AccountInfoDouble(ACCOUNT_BALANCE);
   ObjectSetString(0, "AccountSize", OBJPROP_TEXT, "Acc. " + SizeText + ": " + DoubleToString(Size, 2));

   CalculateRiskAndPositionSize();
}

//+------------------------------------------------------------------+
//| Object dragging handler                                          |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,         // Event ID
                  const long& lparam,   // Parameter of type long event
                  const double& dparam, // Parameter of type double event
                  const string& sparam  // Parameter of type string events
)
{
   if (id != CHARTEVENT_OBJECT_DRAG) return;
   if ((sparam != "EntryLine") && (sparam != "StopLossLine")) return;
   RecalculatePositionSize();
}

int OnCalculate(const int rates_total,      // size of the price[] array
                 const int prev_calculated,  // bars handled on a previous call
                 const int begin,            // where the significant data start from
                 const double& price[]       // array to calculate
)
{
   RecalculatePositionSize();
   return(rates_total);
}

//+------------------------------------------------------------------+
//| Calculates risk size and position size. Sets object values.      |
//+------------------------------------------------------------------+
void CalculateRiskAndPositionSize()
{
   if (!UseMoneyInsteadOfPercentage) RiskMoney = Size * Risk / 100;
   else RiskMoney = MoneyRisk;
   ObjectSetString(0, "RiskMoney", OBJPROP_TEXT, "Risk, money:  " + DoubleToString(RiskMoney, 2));

   double UnitCost = SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_VALUE);
   double TickSize = SymbolInfoDouble(_Symbol, SYMBOL_TRADE_TICK_SIZE);
   if ((StopLoss != 0) && (UnitCost != 0) && (TickSize != 0)) PositionSize = RiskMoney / (StopLoss * UnitCost / TickSize);
   
   ObjectSetString(0, "PositionSize", OBJPROP_TEXT, "Pos. Size:    " + DoubleToString(PositionSize, 2));
}
//+------------------------------------------------------------------+